#!/usr/bin/python
# -*- coding: utf-8 -*-

# PyGtalkRobot: A simple jabber/xmpp bot framework using Regular Expression Pattern as command controller
# Copyright (c) 2008 Demiao Lin <ldmiao@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Homepage: http://code.google.com/p/pygtalkrobot/
#
#
# This is modified version of the original PyGtalkRobot

import sys, traceback
import xmpp
import urllib
import re
import inspect

import os

import alfredo

"""A simple jabber/xmpp bot framework

This is a simple jabber/xmpp bot framework using Regular Expression Pattern as command controller.
Copyright (c) 2008 Demiao Lin <ldmiao@gmail.com>

"""

class GtalkRobot(object):

    ########################################################################################################################
    conn = None
    show = "available"
    status = "PyGtalkRobot"
    ########################################################################################################################
    
    #show : xa,away---away   dnd---busy   available--online
    def setState(self, show, status_text):
        if show:
            show = show.lower()
        if show == "online" or show == "on" or show == "available":
            show = "available"
        elif show == "busy" or show == "dnd":
            show = "dnd"
        elif show == "away" or show == "idle" or show == "off" or show == "out" or show == "xa":
            show = "xa"
        else:
            show = "available"
        
        self.show = show

        if status_text:
            self.status = status_text
        
        if self.conn:
            pres=xmpp.Presence(priority=5, show=self.show, status=self.status)
            self.conn.send(pres)

    def getState(self):
        return self.show, self.status

    def _replyMessage(self, user, message):
        self.conn.send(xmpp.Message(user, message))

    def getRoster(self):
        return self.conn.getRoster()

    def getResources(self, jid):
        roster = self.getRoster()
        if roster:
            return roster.getResources(jid)

    def getShow(self, jid):
        roster = self.getRoster()
        if roster:
            return roster.getShow(jid)

    def getStatus(self, jid):
        roster = self.getRoster()
        if roster:
            return roster.getStatus(jid)

    def authorize(self, jid):
        """ Authorise JID 'jid'. Works only if these JID requested auth previously. """
        self.getRoster().Authorize(jid)
    
    def _processMessage(self, conn, message):
        text = message.getBody()
        user = message.getFrom()
        if text:
            text = text.encode('utf-8', 'ignore')
            command, args = self._parse_message(text)
            found = None
            for implementor in alfredo.ICommand.implementors():
              if implementor.match_name(command):
                found = implementor
            if not found:
              self._replyMessage(user, '{0} is not a valid command. Try help to know more'.format(command))
            else:
              try:
                r = found.run(user, command, *args)
                self._replyMessage(user, r)
              except Exception, e:
                self._replyMessage(user, 'Error running command: {0}.'.format(text))
                print e

    def presenceHandler(self, conn, presence):
        if presence:
            if presence.getType()=='subscribe':
                jid = presence.getFrom().getStripped()
                sys.stderr.write("Subscribe: {0}".format(jid))
                self.authorize(jid)

    '''
     Parses the received message. Returns a tuple (command, (*args))
    '''
    def _parse_message(self, message):
      command = message.split(' ')
      return (command[0], tuple(filter(lambda x: len(x) > 0, command[1:])) )

    def StepOn(self):
        try:
            self.conn.Process(1)
        except KeyboardInterrupt: 
            return 0
        return 1

    def GoOn(self):
        while self.StepOn(): pass

    ########################################################################################################################
    # "debug" parameter specifies the debug IDs that will go into debug output.
    # You can either specifiy an "include" or "exclude" list. The latter is done via adding "always" pseudo-ID to the list.
    # Full list: ['nodebuilder', 'dispatcher', 'gen_auth', 'SASL_auth', 'bind', 'socket', 'CONNECTproxy', 'TLS', 'roster', 'browser', 'ibb'].
    def __init__(self, server_host="talk.google.com", server_port=5223, debug=[]):
        self.debug = debug
        self.server_host = server_host
        self.server_port = server_port

    def start(self, gmail_account, password):
        jid=xmpp.JID(gmail_account)
        user, server, password = jid.getNode(), jid.getDomain(), password
        
        self.conn=xmpp.Client(server, debug=self.debug)
        #talk.google.com
        conres=self.conn.connect( server=(self.server_host, self.server_port) )
        if not conres:
            print "Unable to connect to server %s!"%server
            sys.exit(1)
        if conres<>'tls':
            print "Warning: unable to estabilish secure connection - TLS failed!"
        
        authres=self.conn.auth(user, password)
        if not authres:
            print "Unable to authorize on %s - Plsese check your name/password."%server
            sys.exit(1)
        if authres<>"sasl":
            print "Warning: unable to perform SASL auth os %s. Old authentication method used!"%server
        
        self.conn.RegisterHandler("message", self._processMessage)
        self.conn.RegisterHandler('presence',self.presenceHandler)
        
        self.conn.sendInitPresence()
        
        self.setState(self.show, self.status)
        
        print "Bot started, logged in as {0}".format(user)
        print "IComand implementors found: {0}".format(alfredo.ICommand.implementors())
        self.GoOn()





import alfredo

class HelpCommand(alfredo.Plugin):

  implements = [alfredo.ICommand]
  SHORTHELP = 'List all commands. Type help <command> for more details'

  def name(self):
    return 'help'

  def help(self):
    return (self.SHORTHELP, self.SHORTHELP)

  def match_name(self, command):
    return 'help' == command

  def run(self, user, command, *args):
    if len(args) > 0:
      for implementor in alfredo.ICommand.implementors():
        if implementor.match_name(args[0]):
          return implementor.help()[1]
    return "\n".join(["{0} - {1}".format(c.name(), c.help()[0]) for c in alfredo.ICommand.implementors()])




import base64
import alfredo

class Base64Command(alfredo.Plugin):

  implements = [alfredo.ICommand,]
  SHORTHELP = 'b64d/b64e Decode/Encode form/to Base64'
  LONGHELP = 'b64d decodes from Base64, b64e Encodes to base64'

  def help(self):
    return (self.SHORTHELP, self.LONGHELP)

  def name(self):
    return 'b64e/b64d'

  def match_name(self, command):
    return command in ('b64e', 'b64d')

  def run(self, user, command, *data):
    if len(data) == 0:
      return "{0} command needs one argument".format(command)
    return getattr(self, "_{0}".format(command))(data[0])


  def _b64e(self, data):
    return base64.b64encode(data)

  def _b64d(self, data):
    return base64.b64decode(data)



import alfredo

import requests
from BeautifulSoup import BeautifulSoup, Tag

class MegaUploadCommand(alfredo.Plugin):

  implements = [alfredo.ICommand]

  def help(self):
    return ("Discover the final URL for a megaupload file", "Call mu <URL> to get the final link.")

  def name(self):
    return 'mu'

  def match_name(self, command):
    return 'mu' == command

  def run(self, user, command, *args):
    if len(args) < 1:
      return 'Needs a least one argument'
    return mu(args[0])


def mu(link):
  r = requests.get(link).content
  html = BeautifulSoup(r)
  href = html.findAll('a', attrs = {'class': 'down_butt1'})
  if len(href) > 0:
    return href[0]['href']
  else:
    return "{0} is not a valid Megaupload link anymore".format(link)



#!/usr/bin/env python
# encoding: utf-8
from setuptools import setup
import os


BASE_PATH = os.path.dirname(__file__)

setup(
  name="alfredo",
  version="0.2",
  url="https://github.com/daltonmatos/alfredo",
  license="3-BSD",
  description="Alfredo is a gtalk bot born so serve you.",
  author="Dalton Barreto",
  author_email="daltonmatos@gmail.com",
  long_description=open(os.path.join(BASE_PATH, 'README.rst')).read(),
  packages=['alfredo', 'alfredo/commands'],
  scripts=[os.path.join(BASE_PATH, 'script/alfredo'), os.path.join(BASE_PATH, 'script/ud')],
  install_requires=['BeautifulSoup==3.2.1', 'plugnplay==0.5.0', 'requests==0.13.2', 'simplejson==2.6.0', 'xmpppy==0.5.0rc1', ],
  classifiers=[
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Topic :: Software Development :: Libraries :: Application Frameworks"
    ])



#encoding: utf-8

import os
import sys
from PyGtalkRobot import GtalkRobot

import plugnplay

__all__ = ['ICommand',  'Plugin']

Plugin = plugnplay.Plugin

class ICommand(plugnplay.Interface):

  '''
   Returns tha command name
  '''
  def name(self):
    pass

  '''
   Returns a tuple (short-help, long-help)
   about the command
  '''
  def help(self):
    pass

  '''
   Returns true if the command name passed as a paramater
   matches the command implemented by this interface
  '''
  def match_name(self, command):
    pass

  '''
   Run the specific command. Returns a string.
   Receives the username of the sender and any parameters
   passed on the message, eg:

    sender: process arg0 arg1 arg2

   would be translated to: run(sender, 'process', arg0, arg1, arg2)
  '''
  def run(self, user, command, *args):
    pass

commands_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), 'commands'))
plugnplay.set_plugin_dirs(commands_dir)
plugnplay.load_plugins()






import alfredo

import requests
from BeautifulSoup import BeautifulSoup, Tag
import urllib


URL = "http://www.urbandictionary.com/define.php?"

class UdCommand(alfredo.Plugin):

  implements = [alfredo.ICommand]

  def help(self):
    return ("Searches any term on urbandictionary.com", "Just call ud <term> and you will se the first available definition as returned by urbandictionary, and a link to the full result page.")

  def name(self):
    return 'ud'

  def match_name(self, command):
    return 'ud' == command

  def run(self, user, command, *args):
    if len(args) < 1:
      return 'Needs a least one argument'
    return urban(" ".join(list(args)))



def urban(term):

  params = urllib.urlencode({'term': term})
  response = requests.get(URL + params).content
  html = BeautifulSoup(response)
  tags = html.findAll('div', attrs = {'class': 'definition'})
  if len(tags) >= 1:
    text = ""
    for c in tags[0].contents:
      if isinstance(c, Tag):
        text += unicode("".join(c.contents))
      else:
        text += unicode(c)
    text = text.encode('utf-8')
    return "{0}:\n{1}\nYou can see all results at: {2}".format(term, unescape(text), URL + params)
  return "Term not found: {0}".format(term)


import htmllib

def unescape(s):
    p = htmllib.HTMLParser(None)
    p.save_bgn()
    p.feed(s)
    return p.save_end()



import alfredo

class InvertCommand(alfredo.Plugin):
  implements = [alfredo.ICommand]
  SHORTHELP = 'Inverts all passed arguments'

  def name(self):
    return 'inv'

  def help(self):
    return (self.SHORTHELP, self.SHORTHELP)

  def match_name(self, command):
    return 'inv' == command

  def run(self, user, command, *args):
    if len(args) == 0:
      return "inv command needs at least one argument"
    return " ".join(s[::-1] for s in args[::-1])



#coding: utf-8
import alfredo

class RamalEsys(alfredo.Plugin):

  implements = [alfredo.ICommand]

  def help(self):
    return ("Procura por um ramal", "Apenas chame <ramal> <nome> e você verá uma lista dos ramais encontrados.")

  def name(self):
    return 'ramal'

  def match_name(self, command):
    return 'ramal' == command

  def run(self, user, command, *args):
    if len(args) == 1:
      nome = args[0]
    else:
      nome = None
    
    return ramal(nome)



def ramal(nome=None):

  import simplejson as json
  from os.path import dirname, join
  with open(join(dirname(__file__),'ramais.json')) as f:
    s = f.read()
  
  j = json.loads(s)
  
  res = []
  for reg in j:
    if nome.lower() in reg['nome'].lower():
        res.append(reg)
  
  if len(res) == 0:
    s = u'sem informações para %s' % (nome, )
  else:
    s = '\n\n'.join(['%s (%s) - Ramal: %s Email: %s' % (v['nome'], v['depto'], v['ramal'], v['email']) for v in res])
        
  return s.encode('utf-8')
